/* jslint node: true */

'use strict';

var React = require('react');

var NumeralFormatter = require('./shortcuts/NumeralFormatter');
var DateFormatter = require('./shortcuts/DateFormatter');
var PhoneFormatter = require('./shortcuts/PhoneFormatter');
var CreditCardDetector = require('./shortcuts/CreditCardDetector');

var Utils = {
    strip: function (value, re) {
        return value.replace(re, '');
    },

    headStr: function (str, length) {
        return str.slice(0, length);
    }
};

var Cleave = React.createClass({
    componentDidMount: function () {
        this.init();
    },

    getInitialState: function () {
        var props = this.props,
            vars = {};

        // credit card
        vars.creditCard = !!props.creditCard;
        vars.creditCardStrictMode = !!props.creditCardStrictMode;

        // phone
        vars.phone = !!props.phone;
        vars.phoneRegionCode = props.phoneRegionCode || '';
        vars.phoneFormatter = {};

        // date
        vars.date = !!props.date;
        vars.datePattern = props.datePattern || ['d', 'm', 'Y'];
        vars.dateFormatter = {};

        // numeral
        vars.numeral = !!props.numeral;
        vars.numeralDecimalScale = props.numeralDecimalScale || 2;
        vars.numeralDecimalMark = props.numeralDecimalMark || '.';
        vars.numeralThousandsGroupStyle = props.numeralThousandsGroupStyle || 'thousand';

        vars.numericOnly = vars.creditCard || vars.date || !!props.numericOnly;

        vars.prefix = (vars.creditCard || vars.phone || vars.date) ? '' : (props.prefix || '');
        vars.prefixLength = vars.prefix.length;

        vars.delimiter = props.delimiter || (vars.date ? '/' : (vars.numeral ? ',' : ' '));
        vars.delimiterRE = new RegExp(vars.delimiter, "g");

        vars.blocks = props.blocks || [];
        vars.blocksLength = vars.blocks.length;

        vars.maxLength = 0;

        vars.backspace = false;
        vars.result = '';

        return {
            vars: vars
        };
    },

    init: function () {
        var owner = this,
            vars = owner.state.vars;

        // so no need for this lib at all
        if (!vars.numeral && !vars.phone && !vars.creditCard && !vars.date && vars.blocks.length === 0) {
            return;
        }

        vars.maxLength = owner.getMaxLength();

        owner.initPhoneFormatter();
        owner.initDateFormatter();
        owner.initNumeralFormatter();

        owner.onInput();

        this.forceUpdate();
    },

    initNumeralFormatter: function () {
        var owner = this,
            vars = owner.state.vars;

        if (!vars.numeral) {
            return;
        }

        vars.numeralFormatter = new NumeralFormatter(
            vars.numeralDecimalMark,
            vars.numeralDecimalScale,
            vars.numeralThousandsGroupStyle,
            vars.delimiter
        );

        this.forceUpdate();
    },

    initDateFormatter: function () {
        var owner = this,
            vars = owner.state.vars;

        if (!vars.date) {
            return;
        }

        vars.dateFormatter = new DateFormatter(vars.datePattern);
        vars.blocks = vars.dateFormatter.getBlocks();
        vars.blocksLength = vars.blocks.length;
        vars.maxLength = owner.getMaxLength();

        this.forceUpdate();
    },

    initPhoneFormatter: function () {
        var owner = this,
            vars = owner.state.vars;

        if (!vars.phone) {
            return;
        }

        // Cleave.AsYouTypeFormatter should be provided by
        // external google closure lib
        //try {
        //    vars.phoneFormatter = new PhoneFormatter(
        //        new window.Cleave.AsYouTypeFormatter(vars.phoneRegionCode),
        //        vars.delimiter
        //    );
        //} catch (ex) {
        //    throw new Error('Please include phone-type-formatter.{country}.js lib');
        //}

        this.forceUpdate();
    },

    onKeydown: function (event) {
        var owner = this,
            vars = owner.state.vars,
            charCode = event.which || event.keyCode;

        // hit backspace when last character is delimiter
        if (charCode === 8 && vars.result.slice(-1) === vars.delimiter) {
            vars.backspace = true;

            return;
        }

        vars.backspace = false;

        this.forceUpdate();
    },

    getMaxLength: function () {
        return this.state.vars.blocks.reduce(function (previous, current) {
            return previous + current;
        }, 0);
    },

    onInput: function () {
        var owner = this,
            vars = owner.state.vars,
            value = vars.result,
            prev = value,
            prefixLengthValue,
            result;

        // case 1: delete one more character "4"
        // 1234*| -> hit backspace -> 123|
        // case 2: last character is not delimiter which is:
        // 12|34* -> hit backspace -> 1|34*

        if (owner.backspace && value.slice(-1) !== vars.delimiter) {
            value = Utils.headStr(value, value.length - 1);
        }

        // phone formatter
        if (vars.phone) {
            //vars.result = vars.phoneFormatter.format(value);

            this.forceUpdate();

            return;
        }

        // numeral formatter
        if (vars.numeral) {
            vars.result = vars.numeralFormatter.format(value);

            this.forceUpdate();

            return;
        }

        // date
        if (vars.date) {
            value = vars.dateFormatter.getValidatedDate(value);
        }

        // strip delimiters
        value = Utils.strip(value, vars.delimiterRE);

        // prefix
        if (vars.prefix.length > 0) {
            prefixLengthValue = Utils.headStr(value, vars.prefixLength);

            if (prefixLengthValue.length < vars.prefixLength) {
                value = vars.prefix;
            } else if (prefixLengthValue !== vars.prefix) {
                value = vars.prefix + value.slice(vars.prefixLength);
            }
        }

        // strip non-numeric characters
        if (vars.numericOnly) {
            value = Utils.strip(value, /[^\d]/g);
        }

        // update credit card blocks
        // and at least one of first 4 characters has changed
        if (vars.creditCard && Utils.headStr(vars.result, 4) !== Utils.headStr(value, 4)) {
            vars.blocks = CreditCardDetector.getBlocksByPAN(value, vars.creditCardStrictMode);
            vars.blocksLength = vars.blocks.length;
            vars.maxLength = owner.getMaxLength();
        }

        // strip over length characters
        value = Utils.headStr(value, vars.maxLength);

        // apply blocks
        result = '';

        vars.blocks.forEach(function (length, index) {
            if (value.length > 0) {
                var sub = value.slice(0, length),
                    rest = value.slice(length);

                result += sub;

                if (sub.length === length && index < vars.blocksLength - 1) {
                    result += vars.delimiter;
                }

                // update remaining string
                value = rest;
            }
        });

        if (prev === result) {
            return;
        }

        vars.result = result;

        this.forceUpdate();
    },

    render: function () {
        return (
            <input type="text"
                   value={this.state.vars.result}
                   onKeydown={this.onKeydown}
                   onInput={this.onInput}/>
        );
    }
});

module.exports = Cleave;
